/**
 * Copyright (c) 2001-2022 by PDFTron Systems Inc. All Rights Reserved.
 * Consult legal.txt regarding legal and license information.
 */
#ifndef   H_PDFSECUREAPI
#define   H_PDFSECUREAPI

/** 
 * The calling convention to use for all calls across the "library boundary."
 */
#ifdef _WIN32
  #define PDFSECURE_API_CALL __stdcall
	#ifdef PDFSECURE_API_EXPORT
		#define PDFSECURE_API(type) __declspec(dllexport) type PDFSECURE_API_CALL 
	#else
		#define PDFSECURE_API(type) __declspec(dllimport) type PDFSECURE_API_CALL 
	#endif
#else
  #define PDFSECURE_API_CALL
 	#ifdef PDFSECURE_API_EXPORT
		#define PDFSECURE_API(type) __attribute__ ((visibility("default"))) type PDFSECURE_API_CALL 
	#else
		#define PDFSECURE_API(type) type PDFSECURE_API_CALL 
	#endif
#endif

#ifdef __cplusplus
extern "C" {
#endif

/** 
 * 'mode' identifier passed in PDFSecureCallback.
 * The 'mode' can be used to determine whether a message is an error or 
 * a progress message.
 */ 
#define PDFSECURE_ERROR		   1 /* Show the error message  */
#define PDFSECURE_MSG		   2 /* Report the message      */
#define PDFSECURE_GETPASS	   3 /* Get the password        */
#define PDFSECURE_OUT_FILENAME 4 /* Reports the Saved File Names */

typedef char* PDFSECURE_API_CALL PDFSecureCallback(int mode, char* msg, void* user_data);

/**
 * Initialize PDFSecure with registration and license information.
 * Although PDFSecureInit is typically called only once (e.g. during the program 
 * startup), it can also be called multiple times without side effects. 
 *
 * The first parameter is your username, the second parameter is your company name, 
 * and the last parameter is the license key supplied with your registration information.
 */
PDFSECURE_API(int)
PDFSecureInit(char* user_name, char* company, char* license_key);

/** 
 * Return codes for PDFSecureRun function.
 */ 
#define PDFSECURE_OK  0             /* No errors */
#define PDFSECURE_ERR 1             /* Unspecified error */
#define PDFSECURE_ERR_BADKEY  2     /* Bad license key */
#define PDFSECURE_ERR_DIRCREATE  3  /* Failed to create the output directory */


/**
 * Execute PDFSecure command string.
 * 
 * PDFSecureRun can be called multiple times to process PDF documents or folders 
 * with PDF documents. The function is completely thread safe, so it is possible to 
 * run many PDFSecure threads at the same time.
 * 
 * @param command_str PDFSecure command string. For a detailed explanation of 
 * all options and syntax, please refer to section 4 in PDFSecure SDK Manual. 
 *
 * @param funct (optional) a pointer to the callback function used to report error 
 * as well as other messages. 
 *
 * @param user_data (optional) a pointer to custom data that you may want to 
 * pass to the callback function. NULL if you don't need to pass custom data.
 *
 * @return error code. A non-zero value indicates that an error was encountered.
 */
PDFSECURE_API(int)
PDFSecureRun(char* command_str, PDFSecureCallback* funct, void* user_data);

#ifdef __cplusplus
}
#endif

#endif  /* H_PDFSECUREAPI */
